#include <stdio.h>
#include <strings.h>
#include <stdlib.h>

#include "minishare.h"
#include "globals.h"
#include "template.h"
#include "files.h"
#include "misc.h"
#include "error.h"

typedef struct {
	int id;
	char *name;
} TOKEN;

#define NUM_TOKENS 29

#define STATE_NORMAL 0
#define STATE_FILE_LIST 1

const TOKEN token_list[NUM_TOKENS]={
	{TOKEN_ID_COND_IF,"if"},
	{TOKEN_ID_COND_ELSE,"else"},
	{TOKEN_ID_COND_ENDIF,"elseif"},
	{TOKEN_ID_COND_ENDIF,"endif"},
	
	{TOKEN_ID_VERSION,"minishare.version"},
	{TOKEN_ID_MINISHARE_URL,"minishare.homepage"},
	
	{TOKEN_ID_BEGINFILES,"filelist.begin"},
	{TOKEN_ID_ENDFILES,"filelist.end"},
	{TOKEN_ID_TOTAL_SIZE,"filelist.size"},
	{TOKEN_ID_SHARED_FILES,"filelist.numfiles"},
	
	{TOKEN_ID_FILE_FILENAME,"file.name"},
	{TOKEN_ID_FILE_FILEPATH,"file.path"},
	{TOKEN_ID_FILE_FILESIZE,"file.size"},
	{TOKEN_ID_FILE_FILEDATE,"file.date"},
	{TOKEN_ID_FILE_URL,"file.url"},
	{TOKEN_ID_FILE_FILEDATE,"file.extension"},
	{TOKEN_ID_FILE_FILEMIME,"file.mime"},
	{TOKEN_ID_FILE_FILEID,"file.id"},
	
	{TOKEN_ID_SERVERNAME,"server.name"},
	{TOKEN_ID_PORT,"server.port"},
	{TOKEN_ID_ADDRESS,"server.address"},
	{TOKEN_ID_MAXCONNECTIONS,"server.maxconnections"},
	{TOKEN_ID_CONNECTIONS,"server.numconnections"},
	
	{TOKEN_ID_CURRENTLOGIN,"request.login"},
	{TOKEN_ID_CURRENTPASSWD,"request.password"},
	{TOKEN_ID_CURRENTPATH,"request.path"},
	{TOKEN_ID_REQUESTSTRING,"request.request_string"},
	
	{TOKEN_ID_ERRORCODE,"error.code"},
	{TOKEN_ID_ERRORTEXT,"error.title"}
};

void cond_push(PREPROCESSOR *p,int cond)
{
	p->cond_i++;
	if (p->cond_i>=CONDBUFSIZE) p->cond_i=0;
	p->cond_buf[p->cond_i]=cond;
}

void cond_pop(PREPROCESSOR *p)
{
	p->cond_i--;
	if (p->cond_i<0) p->cond_i=CONDBUFSIZE-1;
}

void cond_not(PREPROCESSOR *p)
{
	p->cond_buf[p->cond_i]^=1;
}


void process_token(PREPROCESSOR *p)
{
	p->inject_buffer[0]='\0';
	
	int i,strip_len;
	char token_name[TEMPLATEBUFFERSIZE+1]="";
	
	if (sscanf(p->token,"%[^: ]:%d",token_name,&strip_len)!=2)
	{	
		sscanf(p->token,"%s",token_name);
		//strcpy(p->inject_buffer,p->token);
		strip_len=INJECTBUFFERSIZE-1;
		//return;
	}
	
	if (strip_len>=INJECTBUFFERSIZE||strip_len<=-INJECTBUFFERSIZE) strip_len=INJECTBUFFERSIZE-1;
	
	if (token_name[0]=='\0') 
	{
		strcpy(p->inject_buffer,"%");
		p->inject_position=0;
		return;
	}
	
	
	
	int token_id=-1;
	for (i=0;(i<NUM_TOKENS);i++) if (stricmp(token_name,token_list[i].name)==0) token_id=token_list[i].id;
	
	
	if (token_id>=TOKEN_ID_COND_IF && token_id<=TOKEN_ID_COND_ENDIF)
	switch (token_id)
	{
		case TOKEN_ID_COND_ELSEIF:
			cond_not(p);
		case TOKEN_ID_COND_IF:
		{
			char test[INJECTBUFFERSIZE+1],var[INJECTBUFFERSIZE+1],imm[INJECTBUFFERSIZE+1];
			if (sscanf(p->token,"if %[^<>=!]%[<>=!]%[^<>=!]",var,test,imm)==3)
			{
				PREPROCESSOR *temp_processor=open_preprocessor(NULL); 	// obviously not very pretty
				memcpy(temp_processor,p,sizeof(PREPROCESSOR));			//
				temp_processor->source=NULL;							// FIXME
				
				if (var[0]!='\"')
				{																		
					strcpy(temp_processor->token,var);
					process_token(temp_processor);
					strcpy(var,temp_processor->inject_buffer);
				} else {
					char temp[INJECTBUFFERSIZE+1]="";
					sscanf(var,"\"%[^\"]\"",temp);	
					strcpy(var,temp);
				}
				
				if (imm[0]!='\"')
				{																		
					strcpy(temp_processor->token,imm);
					process_token(temp_processor);
					strcpy(imm,temp_processor->inject_buffer);
				} else {
					char temp[INJECTBUFFERSIZE+1]="";
					sscanf(imm,"\"%[^\"]\"",temp);	
					strcpy(imm,temp);
				}
				
				close_preprocessor(temp_processor);
				
				int int_imm,int_var;
				if (sscanf(var,"%d",&int_var)==1 && sscanf(imm,"%d",&int_imm)==1) // if both are valid numbers...
				{
					if (strcmp(test,"==")==0||strcmp(test,"=")==0) 
					{
						// equal
						if (int_var==int_imm) 
							cond_push(p,true);
						else 
							cond_push(p,false);
							
						// should be: cond_push(p,(int_var==int_imm)); etc.
					} 
					else if (strcmp(test,"!=")==0 || strcmp(test,"<>")==0) 
					{
						// not equal
						if (int_var!=int_imm) 
							cond_push(p,true);
						else 
							cond_push(p,false);
					}
					else if (strcmp(test,"<")==0)
					{
						// less than
						if (int_var<int_imm) 
							cond_push(p,true);
						else 
							cond_push(p,false);
					}
					else if (strcmp(test,">")==0)
					{
						// greater than	
						if (int_var>int_imm) 
							cond_push(p,true);
						else 
							cond_push(p,false);
					}
					else if (strcmp(test,"<=")==0)
					{
						// less than	or equal
						if (int_var<=int_imm) 
							cond_push(p,true);
						else 
							cond_push(p,false);
					}
					else if (strcmp(test,">=")==0)
					{
						// greater than	or equal
						if (int_var>=int_imm) 
							cond_push(p,true);
						else 
							cond_push(p,false);
					} 
					else goto malformed_if;
				} 
				else 
				{
					if (strcmp(test,"==")==0||strcmp(test,"=")==0) 
					{
						// equal
						if (stricmp(var,imm)==0) 
							cond_push(p,true);
						else 
							cond_push(p,false);
					} 
					else if (stricmp(test,"!=")==0 || strcmp(test,"<>")==0) 
					{
						// not equal
						if (strcmp(var,imm)!=0) 
							cond_push(p,true);
						else 
							cond_push(p,false);
					}
					else if (stricmp(test,"<")==0) 
					{
						// less than (lex)
						if (strcmp(var,imm)<0) 
							cond_push(p,true);
						else 
							cond_push(p,false);
					}
					else if (stricmp(test,">")==0) 
					{
						// greater than (lex)
						if (strcmp(var,imm)>0) 
							cond_push(p,true);
						else 
							cond_push(p,false);
					}
					else if (stricmp(test,"<=")==0) 
					{
						// less than or equal (lex)
						if (strcmp(var,imm)<=0) 
							cond_push(p,true);
						else 
							cond_push(p,false);
					}
					else if (stricmp(test,">=")==0) 
					{
						// greater than or equal (lex)
						if (strcmp(var,imm)>=0) 
							cond_push(p,true);
						else 
							cond_push(p,false);
					}
					else goto malformed_if;
				}
				//strcpy(p->inject_buffer,p->cond_buf[p->cond_i]==0?"false":"true");
			} else {
malformed_if:
				strcpy(p->inject_buffer,"ERROR: Malformed IF-clause\n");
				cond_push(p,false);
			}
			
		}
		break;
		
		case TOKEN_ID_COND_ELSE:
		{
			cond_not(p);
		}
		break;
		
		case TOKEN_ID_COND_ENDIF:
		{
			cond_pop(p);	
			//strcpy(p->inject_buffer,p->cond_buf[p->cond_i]==0?"false":"true");
		}	
	 	break;
	 	
	}
	else
	
	/*if (p->cond_buf[p->cond_i])*/ switch (token_id)
	{
		
		
		case TOKEN_ID_VERSION:
		{
			strcpy(p->inject_buffer,version_string);
		}
		break;
		
		case TOKEN_ID_SERVERNAME:
		{
			strcpy(p->inject_buffer,server_name);
		}
		break;
		
		case TOKEN_ID_ADDRESS:
		{
			strcpy(p->inject_buffer,server_domain);
		}
		break;
		
		case TOKEN_ID_PORT:
		{
			sprintf(p->inject_buffer,"%d",stunnel_port==0?server_port:stunnel_port);
		}
		break;
		
		case TOKEN_ID_MINISHARE_URL:
		{
			strcpy(p->inject_buffer,MINISHARE_URL);
		}
		break;
		
		case TOKEN_ID_BEGINFILES:
		{
			p->state=STATE_FILE_LIST;
			p->current_file=0;
			p->file_id=1;
			
			while (!sharedfiles[p->current_file].visible) p->current_file++;
			
			p->loop_position=p->last_read_position+p->buffer_position; // mark filelist loop start
		}
		break;
		
		case TOKEN_ID_ENDFILES:
		{
			p->state=STATE_NORMAL;
			p->file_id++;
			do 
			{
				p->current_file++;
			} 
			while (!sharedfiles[p->current_file].visible && (p->current_file<shared_files));
			
			if (p->current_file<shared_files)
			{
				fseek(p->source,p->loop_position,SEEK_SET); // jump back to where filelist starts
				memset(p->buffer,0,TEMPLATEBUFFERSIZE); // force buffer reload
			}
		}
		break;
		
		case TOKEN_ID_FILE_FILESIZE:
		{
			if (p->current_file>=shared_files) break;
			sizechar(p->inject_buffer,(sharedfiles[p->current_file].filesize<0)?0:sharedfiles[p->current_file].filesize);
		}
		break;
		
		case TOKEN_ID_FILE_FILEPATH:
		{
			if (p->current_file>=shared_files) break;
			strcpy(p->inject_buffer,sharedfiles[p->current_file].filename);
		}
		break;
		
		case TOKEN_ID_FILE_FILEDATE:
		{
			if (p->current_file>=shared_files) break;
			strcpy(p->inject_buffer,sharedfiles[p->current_file].filedate);
		}
		break;
		
		case TOKEN_ID_FILE_FILENAME:
		{
			if (p->current_file>=shared_files) break;
			int i=strlen(sharedfiles[p->current_file].filename);
			
			do 
			{ 
				i--; 
			} 
			while (i>=0 && sharedfiles[p->current_file].filename[i]!='\\');
			
			strcpy(p->inject_buffer,sharedfiles[p->current_file].filename+i+1);
		}
		break;
		
		case TOKEN_ID_FILE_FILEEXTENSION:
		{
			if (p->current_file>=shared_files) break;
			int i=strlen(sharedfiles[p->current_file].filename);
			
			do 
			{ 
				i--; 
			} 
			while (i>=0 && sharedfiles[p->current_file].filename[i]!='\\' && sharedfiles[p->current_file].filename[i]!='.');
			
			if (sharedfiles[p->current_file].filename[i]=='.') 
				strcpy(p->inject_buffer,sharedfiles[p->current_file].filename+i+1);
			else
				p->inject_buffer[0]='\0';
		}
		break;
		
		case TOKEN_ID_FILE_FILEMIME:
		{
			if (p->current_file>=shared_files) break;
			//getmimetype(sharedfiles[p->current_file].filename,p->inject_buffer);
			strcpy(p->inject_buffer,sharedfiles[p->current_file].mime);
		}
		break;
		
		case TOKEN_ID_FILE_URL:
		{
			if (p->current_file>=shared_files) break;
			strcpy(p->inject_buffer,sharedfiles[p->current_file].shared_as);
		}
		break;
		
		case TOKEN_ID_FILE_FILEID:
		{
			sprintf(p->inject_buffer,"%d",p->file_id);
		}
		break;
		
		case TOKEN_ID_SHARED_FILES:
		{
			int c=0,i;
			
			for (i=0;i<shared_files;i++) if (sharedfiles[i].visible) c++;
			
			sprintf(p->inject_buffer,"%d",c);
		}
		break;
		
		case TOKEN_ID_TOTAL_SIZE:
		{
			int i;
			__int64 total_size=0;
			
			for (i=0;i<shared_files;i++) if (sharedfiles[i].visible) total_size+=(sharedfiles[i].filesize<0)?0:sharedfiles[i].filesize;
			
			sizechar(p->inject_buffer,total_size);
		}
		break;
		
		case TOKEN_ID_ERRORCODE:
		{
			sprintf(p->inject_buffer,"%d",p->statuscode);
		}
		break;
		
		case TOKEN_ID_ERRORTEXT:
		{
			get_error_message(p->statuscode,p->inject_buffer);
		}
		break;
		
		case TOKEN_ID_CURRENTPATH:
		{
			strcpy(p->inject_buffer,p->request->path);
		}
		break;
		
		case TOKEN_ID_REQUESTSTRING:
		{
			strcpy(p->inject_buffer,p->request->request_string);
		}
		break;
		
		
		default:
		{
			sprintf(p->inject_buffer,"ERROR: Unknown token '%s'\n",token_name);
		}
		break;
	}
	
	if (strip_len>=0)
	{
		p->inject_buffer[strip_len]='\0';
	}
	else 
	{
		strip_len=-strip_len;
		memcpy(p->inject_buffer,p->inject_buffer+strlen(p->inject_buffer)-strip_len,strip_len);
		p->inject_buffer[strip_len]='\0';
	}
	
	p->inject_position=0;
}

char template_getchar(PREPROCESSOR *p,int *injected)
{
	char c;
	int read_chars=1;
	
	if (p->inject_buffer[p->inject_position]=='\0')
	{
		if (p->buffer[p->buffer_position]=='\0')
		{
			
			p->last_read_position=ftell(p->source);
			if (!feof(p->source))
			{
				p->buffer_position=0;
				if (fgets(p->buffer,TEMPLATEBUFFERSIZE,p->source)==NULL) read_chars=0;
			}
		}
		
		c=p->buffer[p->buffer_position++];
		
		if (read_chars==0) c='\0';
		
		*injected=0;
	}
	else
	{
		c=p->inject_buffer[p->inject_position++];
		*injected=1;
	}
	
	return c;
}

int if_stack(PREPROCESSOR *p)
{
	int i;
	for (i=0;i<=p->cond_i;i++)
		if (!p->cond_buf[i]) return false;
	return true;
}

int template_getline(char *output,const int maxlen,PREPROCESSOR *p)
{
	int output_position=0;
	
	while ((!feof(p->source))&&(output_position<maxlen))
	{
		int injected;
		char c=template_getchar(p,&injected);
		
		 
		
		if (injected)
		{
			if (if_stack(p)) output[output_position++]=c;
		}
		else
		{
			if (p->in_token)
			{
				if (c=='%')
				{
					p->in_token=0;
					p->token[p->token_position]='\0';
					process_token(p);
				}
				else
				{
					if (p->token_position<MAX_TOKENLEN)
						p->token[p->token_position++]=c;
				}
			}
			else
			{
				if (c=='%')
				{
					p->in_token=1;
					p->token_position=0;
				}
				else 
				{
					if (if_stack(p)) output[output_position++]=c;
				}
			}
		}
	}
	
	output[output_position]='\0';
	
	return (feof(p->source)/*&&p->buffer[p->buffer_position]=='\0'*/)?0:1; // i.e. source file at EOF & buffer used
}

PREPROCESSOR * open_preprocessor(char *filename)
{
	PREPROCESSOR *p=(PREPROCESSOR*)malloc(sizeof(PREPROCESSOR));
	
	memset(p,0,sizeof(PREPROCESSOR));
	
	p->cond_buf[0]=1;
	
	if (filename!=NULL)
	{
		p->source=fopen(filename,"r");
		
		if (p->source==NULL) {
			free(p);
			return NULL;
		}
	} else p->source=NULL;
	
	return p;	
}

void close_preprocessor(PREPROCESSOR *p)
{
	if (p->source!=NULL) fclose(p->source);
	free(p);	
}

